#!/usr/bin/env node

const colorFunctions = require("./colors/colors");
const { styleFunctions, additionalStyles } = require("./colors/styles");
const specialEffects = require("./colors/effects");
const pastelColors = require("./colors/pastelColors");
const {
  additionalColors,
  additionalBackgroundColors,
} = require("./colors/additionalColors");
const gradients = require("./colors/gradient");

const justColors = {
  // Regular colors
  red: colorFunctions.red,
  green: colorFunctions.green,
  blue: colorFunctions.blue,
  yellow: colorFunctions.yellow,
  magenta: colorFunctions.magenta,
  cyan: colorFunctions.cyan,
  white: colorFunctions.white,
  gray: colorFunctions.gray,
  black: colorFunctions.black,
  orange: colorFunctions.orange,
  purple: colorFunctions.purple,
  pink: colorFunctions.pink,
  brown: colorFunctions.brown,
  navy: colorFunctions.navy,
  teal: colorFunctions.teal,
  maroon: colorFunctions.maroon,
  olive: colorFunctions.olive,
  lime: colorFunctions.lime,
  coral: colorFunctions.coral,
  salmon: colorFunctions.salmon,
  gold: colorFunctions.gold,
  indigo: colorFunctions.indigo,
  turquoise: colorFunctions.turquoise,
  violet: colorFunctions.violet,
  plum: colorFunctions.plum,
  khaki: colorFunctions.khaki,
  crimson: colorFunctions.crimson,
  orchid: colorFunctions.orchid,
  sienna: colorFunctions.sienna,
  tan: colorFunctions.tan,
  chocolate: colorFunctions.chocolate,
  peru: colorFunctions.peru,
  indianRed: colorFunctions.indianRed,
  darkGreen: colorFunctions.darkGreen,
  darkBlue: colorFunctions.darkBlue,
  darkRed: colorFunctions.darkRed,
  darkCyan: colorFunctions.darkCyan,
  darkMagenta: colorFunctions.darkMagenta,
  darkOrange: colorFunctions.darkOrange,
  darkViolet: colorFunctions.darkViolet,
  darkGray: colorFunctions.darkGray,
  lightGray: colorFunctions.lightGray,
  lightGreen: colorFunctions.lightGreen,
  lightBlue: colorFunctions.lightBlue,
  lightPink: colorFunctions.lightPink,
  lightSalmon: colorFunctions.lightSalmon,
  lightCoral: colorFunctions.lightCoral,
  lightSkyBlue: colorFunctions.lightSkyBlue,

  // Bright colors
  brightRed: colorFunctions.brightRed,
  brightGreen: colorFunctions.brightGreen,
  brightBlue: colorFunctions.brightBlue,
  brightYellow: colorFunctions.brightYellow,
  brightMagenta: colorFunctions.brightMagenta,
  brightCyan: colorFunctions.brightCyan,
  brightWhite: colorFunctions.brightWhite,

  // Background colors
  bgRed: colorFunctions.bgRed,
  bgGreen: colorFunctions.bgGreen,
  bgBlue: colorFunctions.bgBlue,
  bgYellow: colorFunctions.bgYellow,
  bgMagenta: colorFunctions.bgMagenta,
  bgCyan: colorFunctions.bgCyan,
  bgWhite: colorFunctions.bgWhite,
  bgBlack: colorFunctions.bgBlack,

  // Bright background colors
  bgBrightRed: colorFunctions.bgBrightRed,
  bgBrightGreen: colorFunctions.bgBrightGreen,
  bgBrightBlue: colorFunctions.bgBrightBlue,
  bgBrightYellow: colorFunctions.bgBrightYellow,
  bgBrightMagenta: colorFunctions.bgBrightMagenta,
  bgBrightCyan: colorFunctions.bgBrightCyan,
  bgBrightWhite: colorFunctions.bgBrightWhite,

  // Styles
  ...styleFunctions,

  // Special effects
  rainbow: specialEffects.rainbow,
  zebra: specialEffects.zebra,
  america: specialEffects.america,
  trap: specialEffects.trap,
  random: specialEffects.random,

  // Pastel colors
  pastelRed: pastelColors.pastelRed,
  pastelGreen: pastelColors.pastelGreen,
  pastelBlue: pastelColors.pastelBlue,
  pastelYellow: pastelColors.pastelYellow,
  pastelMagenta: pastelColors.pastelMagenta,
  pastelCyan: pastelColors.pastelCyan,

  // Additional colors
  ...additionalColors,

  // Additional background colors
  ...additionalBackgroundColors,

  // Additional styles
  ...additionalStyles,

  // Gradient effects
  sunsetGradient: gradients.sunsetGradient,
  oceanGradient: gradients.oceanGradient,

  // Semantic colors
  success: (text) => colors.green(text),
  error: (text) => colors.red(text),
  warning: (text) => colors.yellow(text),
  info: (text) => colors.blue(text),
  debug: (text) => colors.magenta(text),

  // Background semantic colors
  bgSuccess: (text) => colors.bgGreen(colors.black(text)),
  bgError: (text) => colors.bgRed(colors.white(text)),
  bgWarning: (text) => colors.bgYellow(colors.black(text)),
  bgInfo: (text) => colors.bgBlue(colors.white(text)),
  bgDebug: (text) => colors.bgMagenta(colors.white(text)),

  // Additional gradient effects
  forestGradient: gradients.forestGradient,
  lavenderGradient: gradients.lavenderGradient,

  // Neon colors
  neonGreen: (text) => `\x1b[38;2;57;255;20m${text}\x1b[0m`,
  neonPink: (text) => `\x1b[38;2;255;20;147m${text}\x1b[0m`,
  neonBlue: (text) => `\x1b[38;2;31;81;255m${text}\x1b[0m`,
  neonOrange: (text) => `\x1b[38;2;255;159;0m${text}\x1b[0m`,
  neonYellow: (text) => `\x1b[38;2;255;255;0m${text}\x1b[0m`,
  neonPurple: (text) => `\x1b[38;2;177;0;255m${text}\x1b[0m`,
  neonCyan: (text) => `\x1b[38;2;0;255;255m${text}\x1b[0m`,
  neonRed: (text) => `\x1b[38;2;255;0;0m${text}\x1b[0m`,
  neonLime: (text) => `\x1b[38;2;0;255;0m${text}\x1b[0m`,
  neonMagenta: (text) => `\x1b[38;2;255;0;255m${text}\x1b[0m`,
  neonTurquoise: (text) => `\x1b[38;2;0;255;204m${text}\x1b[0m`,
  neonSkyBlue: (text) => `\x1b[38;2;0;191;255m${text}\x1b[0m`,
  neonHotPink: (text) => `\x1b[38;2;255;105;180m${text}\x1b[0m`,
  neonGold: (text) => `\x1b[38;2;255;215;0m${text}\x1b[0m`,
  neonLavender: (text) => `\x1b[38;2;230;230;250m${text}\x1b[0m`,
  neonMint: (text) => `\x1b[38;2;189;252;201m${text}\x1b[0m`,
  neonCoral: (text) => `\x1b[38;2;255;127;80m${text}\x1b[0m`,
  neonAqua: (text) => `\x1b[38;2;127;255;212m${text}\x1b[0m`,
  neonFuchsia: (text) => `\x1b[38;2;255;0;128m${text}\x1b[0m`,
  neonSalmon: (text) => `\x1b[38;2;255;160;122m${text}\x1b[0m`,
  neonPeriwinkle: (text) => `\x1b[38;2;204;204;255m${text}\x1b[0m`,
  neonTangerine: (text) => `\x1b[38;2;255;204;0m${text}\x1b[0m`,
  neonElectricBlue: (text) => `\x1b[38;2;125;249;255m${text}\x1b[0m`,
  neonLimeGreen: (text) => `\x1b[38;2;50;205;50m${text}\x1b[0m`,
  neonRaspberry: (text) => `\x1b[38;2;227;11;93m${text}\x1b[0m`,
  neonIndigo: (text) => `\x1b[38;2;75;0;130m${text}\x1b[0m`,
  neonViolet: (text) => `\x1b[38;2;238;130;238m${text}\x1b[0m`,
  neonPeach: (text) => `\x1b[38;2;255;218;185m${text}\x1b[0m`,
  neonTeal: (text) => `\x1b[38;2;0;128;128m${text}\x1b[0m`,
  neonOlive: (text) => `\x1b[38;2;128;128;0m${text}\x1b[0m`,
  neonMaroon: (text) => `\x1b[38;2;128;0;0m${text}\x1b[0m`,
  neonNavy: (text) => `\x1b[38;2;0;0;128m${text}\x1b[0m`,
  neonPlum: (text) => `\x1b[38;2;221;160;221m${text}\x1b[0m`,

  // Metallic colors
  gold: (text) => `\x1b[38;2;255;215;0m${text}\x1b[0m`,
  silver: (text) => `\x1b[38;2;192;192;192m${text}\x1b[0m`,
  bronze: (text) => `\x1b[38;2;205;127;50m${text}\x1b[0m`,
  copper: (text) => `\x1b[38;2;184;115;51m${text}\x1b[0m`,
  brass: (text) => `\x1b[38;2;181;166;66m${text}\x1b[0m`,
  steel: (text) => `\x1b[38;2;138;149;151m${text}\x1b[0m`,
  titanium: (text) => `\x1b[38;2;135;134;129m${text}\x1b[0m`,
  platinum: (text) => `\x1b[38;2;229;228;226m${text}\x1b[0m`,
  pewter: (text) => `\x1b[38;2;132;135;137m${text}\x1b[0m`,
  gunmetal: (text) => `\x1b[38;2;42;52;57m${text}\x1b[0m`,
  chrome: (text) => `\x1b[38;2;220;220;220m${text}\x1b[0m`,
  nickel: (text) => `\x1b[38;2;184;184;184m${text}\x1b[0m`,
  aluminum: (text) => `\x1b[38;2;211;211;211m${text}\x1b[0m`,
  tin: (text) => `\x1b[38;2;210;210;210m${text}\x1b[0m`,
  zinc: (text) => `\x1b[38;2;235;235;235m${text}\x1b[0m`,
  lead: (text) => `\x1b[38;2;159;159;159m${text}\x1b[0m`,
  iron: (text) => `\x1b[38;2;155;155;155m${text}\x1b[0m`,
  cobalt: (text) => `\x1b[38;2;0;71;171m${text}\x1b[0m`,
  mercury: (text) => `\x1b[38;2;230;230;230m${text}\x1b[0m`,
  palladium: (text) => `\x1b[38;2;197;190;183m${text}\x1b[0m`,
  rhodium: (text) => `\x1b[38;2;244;244;244m${text}\x1b[0m`,
  osmium: (text) => `\x1b[38;2;152;152;152m${text}\x1b[0m`,
  iridium: (text) => `\x1b[38;2;233;233;233m${text}\x1b[0m`,
  ruthenium: (text) => `\x1b[38;2;207;207;207m${text}\x1b[0m`,
  rhenium: (text) => `\x1b[38;2;190;190;190m${text}\x1b[0m`,
  tungsten: (text) => `\x1b[38;2;85;85;85m${text}\x1b[0m`,
  molybdenum: (text) => `\x1b[38;2;170;170;170m${text}\x1b[0m`,
  tantalum: (text) => `\x1b[38;2;180;180;180m${text}\x1b[0m`,
  niobium: (text) => `\x1b[38;2;200;200;200m${text}\x1b[0m`,
  zirconium: (text) => `\x1b[38;2;215;215;215m${text}\x1b[0m`,
  vanadium: (text) => `\x1b[38;2;165;165;165m${text}\x1b[0m`,

  // Color combinations
  christmasTheme: (text) => colors.red(colors.bgGreen(text)),

  // Text decorations
  frame: (text) => {
    const lines = text.split("\n");
    const width = Math.max(...lines.map((line) => line.length));
    const top = "╔" + "═".repeat(width + 2) + "╗";
    const bottom = "╚" + "═".repeat(width + 2) + "╝";
    const framedLines = lines.map((line) => "║ " + line.padEnd(width) + " ║");
    return [top, ...framedLines, bottom].join("\n");
  },

  // Emoji decorations
  starry: (text) => "✨ " + text + " ✨",
  hearty: (text) => "❤️ " + text + " ❤️",
  coder: (text) => "👨‍💻 " + text + " 👩‍💻",
  bug: (text) => "🐛 " + text + " 🐞",
  rocket: (text) => "🚀 " + text + " 🚀",
  robot: (text) => "🤖 " + text + " 🤖",
  laptop: (text) => "💻 " + text + " 💻",
  discord: (text) => "🎮 " + text + " 🎮",

  // New effects
  spinner: (text) => {
    const frames = [
      "|/-\\",
      "⠋⠙⠹⠸⠼⠴⠦⠧⠇⠏",
      "⣾⣽⣻⢿⡿⣟⣯⣷",
      "▁▂▃▄▅▆▇█▇▆▅▄▃▂▁",
      "▉▊▋▌▍▎▏▎▍▌▋▊▉",
      "▖▘▝▗",
      "◢◣◤◥",
      "◰◳◲◱",
      "◴◷◶◵",
      "◐◓◑◒",
      "◡◡⊙⊙◠◠",
      "⣷⣯⣟⡿⢿⣻⣽⣾",
    ];

    let frameIndex = 0;
    let charIndex = 0;

    return new Promise((resolve) => {
      const interval = setInterval(() => {
        process.stdout.write("\r" + frames[frameIndex][charIndex] + " " + text);
        charIndex = (charIndex + 1) % frames[frameIndex].length;
        if (charIndex === 0) {
          frameIndex = (frameIndex + 1) % frames.length;
        }
      }, 100);

      setTimeout(() => {
        clearInterval(interval);
        process.stdout.write("\r" + " ".repeat(text.length + 2) + "\r");
        resolve(text);
      }, 3000);
    });
  },

  rainbow: (text) => {
    const colors = [
      "\x1b[31m",
      "\x1b[33m",
      "\x1b[32m",
      "\x1b[36m",
      "\x1b[34m",
      "\x1b[35m",
    ];
    return (
      text
        .split("")
        .map((char, i) => colors[i % colors.length] + char)
        .join("") + "\x1b[0m"
    );
  },

  typewriter: (text, speed = 100) => {
    return new Promise((resolve) => {
      let i = 0;
      const interval = setInterval(() => {
        if (i < text.length) {
          process.stdout.write(text[i]);
          i++;
        } else {
          clearInterval(interval);
          console.log();
          resolve(text);
        }
      }, speed);
    });
  },

  blink: (text, interval = 500) => {
    return new Promise((resolve) => {
      let visible = true;
      const blinkInterval = setInterval(() => {
        if (visible) {
          process.stdout.write("\r" + " ".repeat(text.length));
        } else {
          process.stdout.write("\r" + text);
        }
        visible = !visible;
      }, interval);

      setTimeout(() => {
        clearInterval(blinkInterval);
        process.stdout.write("\r" + text + "\n");
        resolve(text);
      }, 5000);
    });
  },

  gradient: (text, startColor = [255, 0, 0], endColor = [0, 0, 255]) => {
    const steps = text.length;
    return text
      .split("")
      .map((char, i) => {
        const r = Math.round(
          startColor[0] + (endColor[0] - startColor[0]) * (i / steps)
        );
        const g = Math.round(
          startColor[1] + (endColor[1] - startColor[1]) * (i / steps)
        );
        const b = Math.round(
          startColor[2] + (endColor[2] - startColor[2]) * (i / steps)
        );
        return `\x1b[38;2;${r};${g};${b}m${char}\x1b[0m`;
      })
      .join("");
  },
};

module.exports = justColors;
